﻿local L = LibStub("AceLocale-3.0"):GetLocale("DKFight")

function DKFight:CreateFrame()
	-- Anchor
	self.Anchor = CreateFrame("Frame", "DKFightAnchor", UIParent)
	self.Anchor:SetResizable(true)
	self.Anchor:SetMinResize(20, 20)
	self.Anchor:SetMovable(true)
	self.Anchor:SetPoint("CENTER", UIParent, "CENTER")
	self.Anchor:SetHeight(50)
	self.Anchor:SetWidth(100)

    -- Display
	self.Display = CreateFrame("Frame", "DKFightDisplay", self.Anchor)
	self.Display:SetResizable(true)
	self.Display:EnableMouse(true)
	self.Display:SetPoint("TOPLEFT", self.Anchor, "TOPLEFT")
	self.Display:SetPoint("TOPRIGHT", self.Anchor, "TOPRIGHT")
	self.Display:SetPoint("BOTTOMLEFT", self.Anchor, "BOTTOMLEFT")
	self.Display:SetPoint("BOTTOMRIGHT", self.Anchor, "BOTTOMRIGHT")

	-- Background
    local bgFrame = {
		bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		tile = true,
		tileSize = 16,
		edgeSize = 6,
		insets = {left = 1, right = 1, top = 1, bottom = 1}
    }

	self.Display:SetBackdrop(bgFrame)
	self.Display:SetBackdropColor(self.db.profile.colors.background.r,
                                  	self.db.profile.colors.background.g,
                                  	self.db.profile.colors.background.b)
	self.Display:SetBackdropBorderColor(0,0,0)
	
	-- Events for scrolling & stopping sizing/moving.
	self.Display:SetScript("OnMouseDown", function(_, button)
            if button == "LeftButton" and not self.db.profile.locked then
                self.Anchor:StartMoving();
            end
        end)

	self.Display:SetScript("OnMouseUp", function()
            if not self.db.profile.locked then
		self.Anchor:StopMovingOrSizing();		
		self:SetAnchors()
            end
	end)

	-- Grip
	self.Grip = CreateFrame("Button", "DKFightResizeGrip", self.Display)
	self.Grip:SetNormalTexture("Interface\\AddOns\\DKFight\\ResizeGrip")
	self.Grip:SetHighlightTexture("Interface\\AddOns\\DKFight\\ResizeGrip")
	self.Grip:SetWidth(16)
	self.Grip:SetHeight(16)
	
	-- Events for resizing.
	self.Grip:SetScript("OnMouseDown", function()
		if not self.db.profile.locked then
			this:GetParent():GetParent():StartSizing()
			this:GetParent():GetParent().IsMovingOrSizing = true
		end
	end)
	
	self.Grip:SetScript("OnMouseUp", function()
		this:GetParent():GetParent():StopMovingOrSizing()
		this:GetParent():GetParent().IsMovingOrSizing = nil
		self:SetAnchors()
	end)
	
	self.Grip:SetPoint("BOTTOMRIGHT", self.Display, "BOTTOMRIGHT", 1, -1)
	
	--Cooldown Icons
	self.Display.Rime = CreateFrame("Frame","RimeFrame", self.Display)
	self.Display.KillingMachine = CreateFrame("Frame","KillingMachineFrame", self.Display)
	self.Display.RuneStrike = CreateFrame("Frame","RuneStrike", self.Display)
	self.Display.SuddenDoom = CreateFrame("Frame","SuddenDoomFrame", self.Display)
	
	
	self.Display.Rime:SetWidth(50)
	self.Display.Rime:SetHeight(50)
	self.Display.Rime:SetPoint("TOPLEFT", 10, -10)
	--Texture setup
	local t = self.Display.Rime:CreateTexture(nil,"BACKGROUND")
	t:SetTexture("Interface/Icons/Spell_Frost_FreezingBreath")
	t:SetAllPoints(self.Display.Rime)
	t:SetAlpha(.8)
	self.Display.Rime.texture = t
	self.textures["Rime"] = t
	
	self.Display.KillingMachine:SetWidth(50)
	self.Display.KillingMachine:SetHeight(50)
	self.Display.KillingMachine:SetPoint("TOPLEFT", 65, -10)
	--Texture setup
	t = self.Display.KillingMachine:CreateTexture(nil,"BACKGROUND")
	t:SetTexture("Interface/Icons/Inv_Sword_122")
	t:ClearAllPoints()
	t:SetAllPoints(self.Display.KillingMachine)
	self.Display.KillingMachine.texture = t
	self.textures["Killing Machine"] = t
	
	self.Display.SuddenDoom:SetWidth(50)
	self.Display.SuddenDoom:SetHeight(50)
	self.Display.SuddenDoom:SetPoint("TOPLEFT", 120, -10)
	--Texture setup
	t = self.Display.SuddenDoom:CreateTexture(nil,"BACKGROUND")
	t:SetTexture("Interface/Icons/Spell_Shadow_DeathCoil")
	t:SetAllPoints(self.Display.SuddenDoom)
	t:SetAlpha(.8)
	self.Display.SuddenDoom.texture = t
	self.textures["Sudden Doom"] = t

	self.Display.RuneStrike:SetHeight(50)
	self.Display.RuneStrike:SetWidth(50)
	self.Display.RuneStrike:SetPoint("TOPLEFT", 175, -10)
	--Texture setup
	t = self.Display.RuneStrike:CreateTexture(nil,"BACKGROUND")
	t:SetTexture("Interface/Icons/Spell_DeathKnight_DarkConviction")
	t:SetAllPoints(self.Display.RuneStrike)
	t:SetAlpha(.8)
	self.Display.RuneStrike.texture = t
	self.textures["Rune Strike"] = t
	
	-- Update the display
	self:UpdateDisplay()
	self.Anchor:SetScript("OnUpdate", function(this, elapsed)
		self:OnDisplayUpdate(elapsed)
  end)
end

function DKFight:OnDisplayUpdate(elapsed)
	--self.timeSinceDKFight = self.timeSinceDKFight + elapsed
		--self.Display:SetAlpha(self.db.profile.colors.background.a * (1 - ((self.timeSinceDKFight - self.db.profile.colors.faded.a)/5)))
		if self.spellStates["Rime"] then
			self.Display.Rime:SetAlpha(1)
		else
			self.Display.Rime:SetAlpha(0)
		end
		
		if self.spellStates["Killing Machine"] then
			self.Display.KillingMachine:SetAlpha(1)
		else
			self.Display.KillingMachine:SetAlpha(0)
		end
		
		if self.spellStates["Sudden Doom"] then
			self.Display.SuddenDoom:SetAlpha(1)
		else
			self.Display.SuddenDoom:SetAlpha(0)
		end
		
		if self.spellStates["Rune Strike"] then
			self.Display.RuneStrike:SetAlpha(1)
		else
			self.Display.RuneStrike:SetAlpha(0)
		end
end

function DKFight:SetAnchors(useDB)
	local x, y, height, width, relativePoint, relativeTo = nil,nil,nil,nil,nil,nil

	if useDB then
		x = self.db.profile.x or 0
        	y = self.db.profile.y or 0
        	height = self.db.profile.h or 50
        	width = self.db.profile.w or 100
        	relTo = self.db.profile.relTo or "CENTER"
        	relPt = self.db.profile.relPt or "CENTER"
	else
        	height, width = self.Anchor:GetHeight(), self.Anchor:GetWidth()
        	relTo,_,relPt,x,y = self.Anchor:GetPoint()
	end

	self.Anchor:ClearAllPoints()
    	self.Anchor:SetPoint(relTo, UIParent, relPt, x, y)
	self.Anchor:SetHeight(height)
	self.Anchor:SetWidth(width)

	self.db.profile.x = x
	self.db.profile.y = y
	self.db.profile.h = h
	self.db.profile.w = w
	self.db.profile.relTo = relTo
	self.db.profile.relPt = relPt
end

function DKFight:UpdateDisplay()
	if not self.Anchor or not self.db.profile.show then return end

	self.Anchor:SetScale(self.db.profile.scale)
	self.Anchor:SetFrameStrata(self.db.profile.strata)

	if self.db.profile.locked then
		self.Grip:Hide()
	else
		self.Grip:Show()
	end

	local color = self.db.profile.colors.background;
	self.Display:SetBackdropColor(color.r, color.g, color.b)
end

function DKFight:UpdateVisible()
	if not self.Anchor then return end

	if self.db.profile.enabled and self.db.profile.show then
		self.Anchor:Show()
	else
		self.Anchor:Hide()
	end
end